@@ -13,23 +13,37 @@
 # permissions and limitations under the License.
 
 """Tools to parse mongo client options."""
+from __future__ import annotations
+
+from typing import TYPE_CHECKING, Any, Mapping, Optional, Sequence, cast
 
 from bson.codec_options import _parse_codec_options
 from pymongo import common
-from pymongo.auth import _build_credentials_tuple
-from pymongo.common import validate_boolean
+from pymongo.auth import MongoCredential, _build_credentials_tuple
 from pymongo.compression_support import CompressionSettings
 from pymongo.errors import ConfigurationError
-from pymongo.monitoring import _EventListeners
+from pymongo.monitoring import _EventListener, _EventListeners
 from pymongo.pool import PoolOptions
 from pymongo.read_concern import ReadConcern
-from pymongo.read_preferences import make_read_preference, read_pref_mode_from_name
+from pymongo.read_preferences import (
+    _ServerMode,
+    make_read_preference,
+    read_pref_mode_from_name,
+)
 from pymongo.server_selectors import any_server_selector
 from pymongo.ssl_support import get_ssl_context
-from pymongo.write_concern import WriteConcern
+from pymongo.write_concern import WriteConcern, validate_boolean
+
+if TYPE_CHECKING:
+    from bson.codec_options import CodecOptions
+    from pymongo.encryption_options import AutoEncryptionOpts
+    from pymongo.pyopenssl_context import SSLContext
+    from pymongo.topology_description import _ServerSelector
 
 
-def _parse_credentials(username, password, database, options):
+def _parse_credentials(
+    username: str, password: str, database: Optional[str], options: Mapping[str, Any]
+) -> Optional[MongoCredential]:
     """Parse authentication credentials."""
     mechanism = options.get("authmechanism", "DEFAULT" if username else None)
     source = options.get("authsource")
@@ -38,7 +52,7 @@ def _parse_credentials(username, password, database, options):
     return None
 
 
-def _parse_read_preference(options):
+def _parse_read_preference(options: Mapping[str, Any]) -> _ServerMode:
     """Parse read preference options."""
     if "read_preference" in options:
         return options["read_preference"]
@@ -50,7 +64,7 @@ def _parse_read_preference(options):
     return make_read_preference(mode, tags, max_staleness)
 
 
-def _parse_write_concern(options):
+def _parse_write_concern(options: Mapping[str, Any]) -> WriteConcern:
     """Parse write concern options."""
     concern = options.get("w")
     wtimeout = options.get("wtimeoutms")
@@ -59,13 +73,13 @@ def _parse_write_concern(options):
     return WriteConcern(concern, wtimeout, j, fsync)
 
 
-def _parse_read_concern(options):
+def _parse_read_concern(options: Mapping[str, Any]) -> ReadConcern:
     """Parse read concern options."""
     concern = options.get("readconcernlevel")
     return ReadConcern(concern)
 
 
-def _parse_ssl_options(options):
+def _parse_ssl_options(options: Mapping[str, Any]) -> tuple[Optional[SSLContext], bool]:
     """Parse ssl options."""
     use_tls = options.get("tls")
     if use_tls is not None:
@@ -124,7 +138,9 @@ def _parse_ssl_options(options):
     return None, allow_invalid_hostnames
 
 
-def _parse_pool_options(username, password, database, options):
+def _parse_pool_options(
+    username: str, password: str, database: Optional[str], options: Mapping[str, Any]
+) -> PoolOptions:
     """Parse connection pool options."""
     credentials = _parse_credentials(username, password, database, options)
     max_pool_size = options.get("maxpoolsize", common.MAX_POOL_SIZE)
@@ -135,7 +151,7 @@ def _parse_pool_options(username, password, database, options):
     connect_timeout = options.get("connecttimeoutms", common.CONNECT_TIMEOUT)
     socket_timeout = options.get("sockettimeoutms")
     wait_queue_timeout = options.get("waitqueuetimeoutms", common.WAIT_QUEUE_TIMEOUT)
-    event_listeners = options.get("event_listeners")
+    event_listeners = cast(Optional[Sequence[_EventListener]], options.get("event_listeners"))
     appname = options.get("appname")
     driver = options.get("driver")
     server_api = options.get("server_api")
@@ -165,7 +181,7 @@ def _parse_pool_options(username, password, database, options):
     )
 
 
-class ClientOptions(object):
+class ClientOptions:
     """Read only configuration options for a MongoClient.
 
     Should not be instantiated directly by application developers. Access
@@ -173,7 +189,9 @@ class ClientOptions(object):
     instead.
     """
 
-    def __init__(self, username, password, database, options):
+    def __init__(
+        self, username: str, password: str, database: Optional[str], options: Mapping[str, Any]
+    ):
         self.__options = options
         self.__codec_options = _parse_codec_options(options)
         self.__direct_connection = options.get("directconnection")
@@ -195,97 +213,118 @@ class ClientOptions(object):
         self.__server_selector = options.get("server_selector", any_server_selector)
         self.__auto_encryption_opts = options.get("auto_encryption_opts")
         self.__load_balanced = options.get("loadbalanced")
+        self.__timeout = options.get("timeoutms")
+        self.__server_monitoring_mode = options.get(
+            "servermonitoringmode", common.SERVER_MONITORING_MODE
+        )
 
     @property
-    def _options(self):
+    def _options(self) -> Mapping[str, Any]:
         """The original options used to create this ClientOptions."""
         return self.__options
 
     @property
-    def connect(self):
+    def connect(self) -> Optional[bool]:
         """Whether to begin discovering a MongoDB topology automatically."""
         return self.__connect
 
     @property
-    def codec_options(self):
+    def codec_options(self) -> CodecOptions:
         """A :class:`~bson.codec_options.CodecOptions` instance."""
         return self.__codec_options
 
     @property
-    def direct_connection(self):
+    def direct_connection(self) -> Optional[bool]:
         """Whether to connect to the deployment in 'Single' topology."""
         return self.__direct_connection
 
     @property
-    def local_threshold_ms(self):
+    def local_threshold_ms(self) -> int:
         """The local threshold for this instance."""
         return self.__local_threshold_ms
 
     @property
-    def server_selection_timeout(self):
+    def server_selection_timeout(self) -> int:
         """The server selection timeout for this instance in seconds."""
         return self.__server_selection_timeout
 
     @property
-    def server_selector(self):
+    def server_selector(self) -> _ServerSelector:
         return self.__server_selector
 
     @property
-    def heartbeat_frequency(self):
+    def heartbeat_frequency(self) -> int:
         """The monitoring frequency in seconds."""
         return self.__heartbeat_frequency
 
     @property
-    def pool_options(self):
+    def pool_options(self) -> PoolOptions:
         """A :class:`~pymongo.pool.PoolOptions` instance."""
         return self.__pool_options
 
     @property
-    def read_preference(self):
+    def read_preference(self) -> _ServerMode:
         """A read preference instance."""
         return self.__read_preference
 
     @property
-    def replica_set_name(self):
+    def replica_set_name(self) -> Optional[str]:
         """Replica set name or None."""
         return self.__replica_set_name
 
     @property
-    def write_concern(self):
+    def write_concern(self) -> WriteConcern:
         """A :class:`~pymongo.write_concern.WriteConcern` instance."""
         return self.__write_concern
 
     @property
-    def read_concern(self):
+    def read_concern(self) -> ReadConcern:
         """A :class:`~pymongo.read_concern.ReadConcern` instance."""
         return self.__read_concern
 
     @property
-    def retry_writes(self):
+    def timeout(self) -> Optional[float]:
+        """The configured timeoutMS converted to seconds, or None.
+
+        .. versionadded:: 4.2
+        """
+        return self.__timeout
+
+    @property
+    def retry_writes(self) -> bool:
         """If this instance should retry supported write operations."""
         return self.__retry_writes
 
     @property
-    def retry_reads(self):
+    def retry_reads(self) -> bool:
         """If this instance should retry supported read operations."""
         return self.__retry_reads
 
     @property
-    def auto_encryption_opts(self):
+    def auto_encryption_opts(self) -> Optional[AutoEncryptionOpts]:
         """A :class:`~pymongo.encryption.AutoEncryptionOpts` or None."""
         return self.__auto_encryption_opts
 
     @property
-    def load_balanced(self):
+    def load_balanced(self) -> Optional[bool]:
         """True if the client was configured to connect to a load balancer."""
         return self.__load_balanced
 
     @property
-    def event_listeners(self):
+    def event_listeners(self) -> list[_EventListeners]:
         """The event listeners registered for this client.
 
         See :mod:`~pymongo.monitoring` for details.
 
         .. versionadded:: 4.0
         """
+        assert self.__pool_options._event_listeners is not None
         return self.__pool_options._event_listeners.event_listeners()
+
+    @property
+    def server_monitoring_mode(self) -> str:
+        """The configured serverMonitoringMode option.
+
+        .. versionadded:: 4.5
+        """
+        return self.__server_monitoring_mode
