@@ -14,24 +14,25 @@
 
 
 """Functions and classes common to multiple pymongo modules."""
+from __future__ import annotations
 
 import datetime
-import inspect
 import warnings
 from collections import OrderedDict, abc
+from difflib import get_close_matches
 from typing import (
+    TYPE_CHECKING,
     Any,
     Callable,
-    Dict,
-    List,
+    Iterator,
     Mapping,
     MutableMapping,
     NoReturn,
     Optional,
     Sequence,
-    Tuple,
     Type,
     Union,
+    overload,
 )
 from urllib.parse import unquote_plus
 
@@ -40,6 +41,7 @@ from bson.binary import UuidRepresentation
 from bson.codec_options import CodecOptions, DatetimeConversion, TypeRegistry
 from bson.raw_bson import RawBSONDocument
 from pymongo.auth import MECHANISMS
+from pymongo.auth_oidc import OIDCCallback
 from pymongo.compression_support import (
     validate_compressors,
     validate_zlib_compression_level,
@@ -52,6 +54,9 @@ from pymongo.read_preferences import _MONGOS_MODES, _ServerMode
 from pymongo.server_api import ServerApi
 from pymongo.write_concern import DEFAULT_WRITE_CONCERN, WriteConcern, validate_boolean
 
+if TYPE_CHECKING:
+    from pymongo.client_session import ClientSession
+
 ORDERED_TYPES: Sequence[Type] = (SON, OrderedDict)
 
 # Defaults until we connect to a server and get updated limits.
@@ -131,8 +136,11 @@ _MAX_END_SESSIONS = 10000
 # Default value for srvServiceName
 SRV_SERVICE_NAME = "mongodb"
 
+# Default value for serverMonitoringMode
+SERVER_MONITORING_MODE = "auto"  # poll/stream/auto
 
-def partition_node(node: str) -> Tuple[str, int]:
+
+def partition_node(node: str) -> tuple[str, int]:
     """Split a host:port string into (host, int(port)) pair."""
     host = node
     port = 27017
@@ -144,7 +152,7 @@ def partition_node(node: str) -> Tuple[str, int]:
     return host, port
 
 
-def clean_node(node: str) -> Tuple[str, int]:
+def clean_node(node: str) -> tuple[str, int]:
     """Split and normalize a node name from a hello response."""
     host, port = partition_node(node)
 
@@ -155,9 +163,12 @@ def clean_node(node: str) -> Tuple[str, int]:
     return host.lower(), port
 
 
-def raise_config_error(key: str, dummy: Any) -> NoReturn:
+def raise_config_error(key: str, suggestions: Optional[list] = None) -> NoReturn:
     """Raise ConfigurationError with the given key name."""
-    raise ConfigurationError(f"Unknown option {key}")
+    msg = f"Unknown option: {key}."
+    if suggestions:
+        msg += f" Did you mean one of ({', '.join(suggestions)}) or maybe a camelCase version of one? Refer to docstring."
+    raise ConfigurationError(msg)
 
 
 # Mapping of URI uuid representation options to valid subtypes.
@@ -187,7 +198,7 @@ def validate_integer(option: str, value: Any) -> int:
         try:
             return int(value)
         except ValueError:
-            raise ValueError(f"The value of {option} must be an integer")
+            raise ValueError(f"The value of {option} must be an integer") from None
     raise TypeError(f"Wrong type for {option}, value must be an integer")
 
 
@@ -279,9 +290,9 @@ def validate_positive_float(option: str, value: Any) -> float:
     try:
         value = float(value)
     except ValueError:
-        raise ValueError(errmsg)
+        raise ValueError(errmsg) from None
     except TypeError:
-        raise TypeError(errmsg)
+        raise TypeError(errmsg) from None
 
     # float('inf') doesn't work in 2.4 or 2.5 on Windows, so just cap floats at
     # one billion - this is a reasonable approximation for infinity
@@ -380,18 +391,18 @@ def validate_uuid_representation(dummy: Any, value: Any) -> int:
         return _UUID_REPRESENTATIONS[value]
     except KeyError:
         raise ValueError(
-            "{} is an invalid UUID representation. "
+            f"{value} is an invalid UUID representation. "
             "Must be one of "
-            "{}".format(value, tuple(_UUID_REPRESENTATIONS))
-        )
+            f"{tuple(_UUID_REPRESENTATIONS)}"
+        ) from None
 
 
-def validate_read_preference_tags(name: str, value: Any) -> List[Dict[str, str]]:
+def validate_read_preference_tags(name: str, value: Any) -> list[dict[str, str]]:
     """Parse readPreferenceTags if passed as a client kwarg."""
     if not isinstance(value, list):
         value = [value]
 
-    tag_sets: List = []
+    tag_sets: list = []
     for tag_set in value:
         if tag_set == "":
             tag_sets.append({})
@@ -403,7 +414,7 @@ def validate_read_preference_tags(name: str, value: Any) -> List[Dict[str, str]]
                 tags[unquote_plus(key)] = unquote_plus(val)
             tag_sets.append(tags)
         except Exception:
-            raise ValueError(f"{tag_set!r} not a valid value for {name}")
+            raise ValueError(f"{tag_set!r} not a valid value for {name}") from None
     return tag_sets
 
 
@@ -413,64 +424,50 @@ _MECHANISM_PROPS = frozenset(
         "CANONICALIZE_HOST_NAME",
         "SERVICE_REALM",
         "AWS_SESSION_TOKEN",
-        "PROVIDER_NAME",
+        "ENVIRONMENT",
+        "TOKEN_RESOURCE",
     ]
 )
 
 
-def validate_auth_mechanism_properties(option: str, value: Any) -> Dict[str, Union[bool, str]]:
+def validate_auth_mechanism_properties(option: str, value: Any) -> dict[str, Union[bool, str]]:
     """Validate authMechanismProperties."""
-    props: Dict[str, Any] = {}
+    props: dict[str, Any] = {}
     if not isinstance(value, str):
         if not isinstance(value, dict):
             raise ValueError("Auth mechanism properties must be given as a string or a dictionary")
-        for key, value in value.items():
+        for key, value in value.items():  # noqa: B020
             if isinstance(value, str):
                 props[key] = value
             elif isinstance(value, bool):
                 props[key] = str(value).lower()
-            elif key in ["allowed_hosts"] and isinstance(value, list):
+            elif key in ["ALLOWED_HOSTS"] and isinstance(value, list):
                 props[key] = value
-            elif inspect.isfunction(value):
-                signature = inspect.signature(value)
-                if key == "request_token_callback":
-                    expected_params = 2
-                elif key == "refresh_token_callback":
-                    expected_params = 2
-                else:
-                    raise ValueError(f"Unrecognized Auth mechanism function {key}")
-                if len(signature.parameters) != expected_params:
-                    msg = f"{key} must accept {expected_params} parameters"
-                    raise ValueError(msg)
+            elif key in ["OIDC_CALLBACK", "OIDC_HUMAN_CALLBACK"]:
+                if not isinstance(value, OIDCCallback):
+                    raise ValueError("callback must be an OIDCCallback object")
                 props[key] = value
             else:
-                raise ValueError(
-                    "Auth mechanism property values must be strings or callback functions"
-                )
+                raise ValueError(f"Invalid type for auth mechanism property {key}, {type(value)}")
         return props
 
     value = validate_string(option, value)
     for opt in value.split(","):
-        try:
-            key, val = opt.split(":")
-        except ValueError:
-            # Try not to leak the token.
-            if "AWS_SESSION_TOKEN" in opt:
-                opt = (
-                    "AWS_SESSION_TOKEN:<redacted token>, did you forget "
-                    "to percent-escape the token with quote_plus?"
-                )
-            raise ValueError(
-                "auth mechanism properties must be "
-                "key:value pairs like SERVICE_NAME:"
-                "mongodb, not {}.".format(opt)
-            )
+        key, _, val = opt.partition(":")
         if key not in _MECHANISM_PROPS:
+            # Try not to leak the token.
+            if "AWS_SESSION_TOKEN" in key:
+                raise ValueError(
+                    "auth mechanism properties must be "
+                    "key:value pairs like AWS_SESSION_TOKEN:<token>"
+                )
+
             raise ValueError(
-                "{} is not a supported auth "
+                f"{key} is not a supported auth "
                 "mechanism property. Must be one of "
-                "{}.".format(key, tuple(_MECHANISM_PROPS))
+                f"{tuple(_MECHANISM_PROPS)}."
             )
+
         if key == "CANONICALIZE_HOST_NAME":
             props[key] = validate_boolean_or_string(key, val)
         else:
@@ -493,9 +490,9 @@ def validate_document_class(
             is_mapping = issubclass(value.__origin__, abc.MutableMapping)
     if not is_mapping and not issubclass(value, RawBSONDocument):
         raise TypeError(
-            "{} must be dict, bson.son.SON, "
+            f"{option} must be dict, bson.son.SON, "
             "bson.raw_bson.RawBSONDocument, or a "
-            "subclass of collections.MutableMapping".format(option)
+            "subclass of collections.MutableMapping"
         )
     return value
 
@@ -507,14 +504,14 @@ def validate_type_registry(option: Any, value: Any) -> Optional[TypeRegistry]:
     return value
 
 
-def validate_list(option: str, value: Any) -> List:
+def validate_list(option: str, value: Any) -> list:
     """Validates that 'value' is a list."""
     if not isinstance(value, list):
         raise TypeError(f"{option} must be a list")
     return value
 
 
-def validate_list_or_none(option: Any, value: Any) -> Optional[List]:
+def validate_list_or_none(option: Any, value: Any) -> Optional[list]:
     """Validates that 'value' is a list or None."""
     if value is None:
         return value
@@ -525,9 +522,9 @@ def validate_list_or_mapping(option: Any, value: Any) -> None:
     """Validates that 'value' is a list or a document."""
     if not isinstance(value, (abc.Mapping, list)):
         raise TypeError(
-            "{} must either be a list or an instance of dict, "
+            f"{option} must either be a list or an instance of dict, "
             "bson.son.SON, or any other type that inherits from "
-            "collections.Mapping".format(option)
+            "collections.Mapping"
         )
 
 
@@ -535,9 +532,9 @@ def validate_is_mapping(option: str, value: Any) -> None:
     """Validate the type of method arguments that expect a document."""
     if not isinstance(value, abc.Mapping):
         raise TypeError(
-            "{} must be an instance of dict, bson.son.SON, or "
+            f"{option} must be an instance of dict, bson.son.SON, or "
             "any other type that inherits from "
-            "collections.Mapping".format(option)
+            "collections.Mapping"
         )
 
 
@@ -545,10 +542,10 @@ def validate_is_document_type(option: str, value: Any) -> None:
     """Validate the type of method arguments that expect a MongoDB document."""
     if not isinstance(value, (abc.MutableMapping, RawBSONDocument)):
         raise TypeError(
-            "{} must be an instance of dict, bson.son.SON, "
+            f"{option} must be an instance of dict, bson.son.SON, "
             "bson.raw_bson.RawBSONDocument, or "
             "a type that inherits from "
-            "collections.MutableMapping".format(option)
+            "collections.MutableMapping"
         )
 
 
@@ -620,9 +617,9 @@ def validate_unicode_decode_error_handler(dummy: Any, value: str) -> str:
     """Validate the Unicode decode error handler option of CodecOptions."""
     if value not in _UNICODE_DECODE_ERROR_HANDLERS:
         raise ValueError(
-            "{} is an invalid Unicode decode error handler. "
+            f"{value} is an invalid Unicode decode error handler. "
             "Must be one of "
-            "{}".format(value, tuple(_UNICODE_DECODE_ERROR_HANDLERS))
+            f"{tuple(_UNICODE_DECODE_ERROR_HANDLERS)}"
         )
     return value
 
@@ -661,9 +658,18 @@ def validate_datetime_conversion(option: Any, value: Any) -> Optional[DatetimeCo
     raise TypeError(f"{option} must be a str or int representing DatetimeConversion")
 
 
+def validate_server_monitoring_mode(option: str, value: str) -> str:
+    """Validate the serverMonitoringMode option."""
+    if value not in {"auto", "stream", "poll"}:
+        raise ValueError(
+            f'{option}={value!r} is invalid. Must be one of "auto", "stream", or "poll"'
+        )
+    return value
+
+
 # Dictionary where keys are the names of public URI options, and values
 # are lists of aliases for that option.
-URI_OPTIONS_ALIAS_MAP: Dict[str, List[str]] = {
+URI_OPTIONS_ALIAS_MAP: dict[str, list[str]] = {
     "tls": ["ssl"],
 }
 
@@ -671,7 +677,7 @@ URI_OPTIONS_ALIAS_MAP: Dict[str, List[str]] = {
 # are functions that validate user-input values for that option. If an option
 # alias uses a different validator than its public counterpart, it should be
 # included here as a key, value pair.
-URI_OPTIONS_VALIDATOR_MAP: Dict[str, Callable[[Any, Any], Any]] = {
+URI_OPTIONS_VALIDATOR_MAP: dict[str, Callable[[Any, Any], Any]] = {
     "appname": validate_appname_or_none,
     "authmechanism": validate_auth_mechanism,
     "authmechanismproperties": validate_auth_mechanism_properties,
@@ -709,11 +715,12 @@ URI_OPTIONS_VALIDATOR_MAP: Dict[str, Callable[[Any, Any], Any]] = {
     "srvservicename": validate_string,
     "srvmaxhosts": validate_non_negative_integer,
     "timeoutms": validate_timeoutms,
+    "servermonitoringmode": validate_server_monitoring_mode,
 }
 
 # Dictionary where keys are the names of URI options specific to pymongo,
 # and values are functions that validate user-input values for those options.
-NONSPEC_OPTIONS_VALIDATOR_MAP: Dict[str, Callable[[Any, Any], Any]] = {
+NONSPEC_OPTIONS_VALIDATOR_MAP: dict[str, Callable[[Any, Any], Any]] = {
     "connect": validate_boolean_or_string,
     "driver": validate_driver_or_none,
     "server_api": validate_server_api_or_none,
@@ -731,7 +738,7 @@ NONSPEC_OPTIONS_VALIDATOR_MAP: Dict[str, Callable[[Any, Any], Any]] = {
 # Dictionary where keys are the names of keyword-only options for the
 # MongoClient constructor, and values are functions that validate user-input
 # values for those options.
-KW_VALIDATORS: Dict[str, Callable[[Any, Any], Any]] = {
+KW_VALIDATORS: dict[str, Callable[[Any, Any], Any]] = {
     "document_class": validate_document_class,
     "type_registry": validate_type_registry,
     "read_preference": validate_read_preference,
@@ -748,14 +755,14 @@ KW_VALIDATORS: Dict[str, Callable[[Any, Any], Any]] = {
 # internally-used names of that URI option. Options with only one name
 # variant need not be included here. Options whose public and internal
 # names are the same need not be included here.
-INTERNAL_URI_OPTION_NAME_MAP: Dict[str, str] = {
+INTERNAL_URI_OPTION_NAME_MAP: dict[str, str] = {
     "ssl": "tls",
 }
 
 # Map from deprecated URI option names to a tuple indicating the method of
 # their deprecation and any additional information that may be needed to
 # construct the warning message.
-URI_OPTIONS_DEPRECATION_MAP: Dict[str, Tuple[str, str]] = {
+URI_OPTIONS_DEPRECATION_MAP: dict[str, tuple[str, str]] = {
     # format: <deprecated option name>: (<mode>, <message>),
     # Supported <mode> values:
     # - 'renamed': <message> should be the new option name. Note that case is
@@ -774,11 +781,11 @@ for optname, aliases in URI_OPTIONS_ALIAS_MAP.items():
             URI_OPTIONS_VALIDATOR_MAP[alias] = URI_OPTIONS_VALIDATOR_MAP[optname]
 
 # Map containing all URI option and keyword argument validators.
-VALIDATORS: Dict[str, Callable[[Any, Any], Any]] = URI_OPTIONS_VALIDATOR_MAP.copy()
+VALIDATORS: dict[str, Callable[[Any, Any], Any]] = URI_OPTIONS_VALIDATOR_MAP.copy()
 VALIDATORS.update(KW_VALIDATORS)
 
 # List of timeout-related options.
-TIMEOUT_OPTIONS: List[str] = [
+TIMEOUT_OPTIONS: list[str] = [
     "connecttimeoutms",
     "heartbeatfrequencyms",
     "maxidletimems",
@@ -788,22 +795,31 @@ TIMEOUT_OPTIONS: List[str] = [
     "waitqueuetimeoutms",
 ]
 
-
 _AUTH_OPTIONS = frozenset(["authmechanismproperties"])
 
 
-def validate_auth_option(option: str, value: Any) -> Tuple[str, Any]:
+def validate_auth_option(option: str, value: Any) -> tuple[str, Any]:
     """Validate optional authentication parameters."""
     lower, value = validate(option, value)
     if lower not in _AUTH_OPTIONS:
-        raise ConfigurationError(f"Unknown authentication option: {option}")
+        raise ConfigurationError(f"Unknown option: {option}. Must be in {_AUTH_OPTIONS}")
     return option, value
 
 
-def validate(option: str, value: Any) -> Tuple[str, Any]:
+def _get_validator(
+    key: str, validators: dict[str, Callable[[Any, Any], Any]], normed_key: Optional[str] = None
+) -> Callable:
+    normed_key = normed_key or key
+    try:
+        return validators[normed_key]
+    except KeyError:
+        suggestions = get_close_matches(normed_key, validators, cutoff=0.2)
+        raise_config_error(key, suggestions)
+
+
+def validate(option: str, value: Any) -> tuple[str, Any]:
     """Generic validation function."""
-    lower = option.lower()
-    validator = VALIDATORS.get(lower, raise_config_error)
+    validator = _get_validator(option, VALIDATORS, normed_key=option.lower())
     value = validator(option, value)
     return option, value
 
@@ -814,42 +830,50 @@ def get_validated_options(
     """Validate each entry in options and raise a warning if it is not valid.
     Returns a copy of options with invalid entries removed.
 
-    :Parameters:
-        - `opts`: A dict containing MongoDB URI options.
-        - `warn` (optional): If ``True`` then warnings will be logged and
+    :param opts: A dict containing MongoDB URI options.
+    :param warn: If ``True`` then warnings will be logged and
           invalid options will be ignored. Otherwise, invalid options will
           cause errors.
     """
     validated_options: MutableMapping[str, Any]
     if isinstance(options, _CaseInsensitiveDictionary):
         validated_options = _CaseInsensitiveDictionary()
-        get_normed_key = lambda x: x  # noqa: E731
-        get_setter_key = lambda x: options.cased_key(x)  # noqa: E731
+
+        def get_normed_key(x: str) -> str:
+            return x
+
+        def get_setter_key(x: str) -> str:
+            return options.cased_key(x)  # type: ignore[attr-defined]
+
     else:
         validated_options = {}
-        get_normed_key = lambda x: x.lower()  # noqa: E731
-        get_setter_key = lambda x: x  # noqa: E731
+
+        def get_normed_key(x: str) -> str:
+            return x.lower()
+
+        def get_setter_key(x: str) -> str:
+            return x
 
     for opt, value in options.items():
         normed_key = get_normed_key(opt)
         try:
-            validator = URI_OPTIONS_VALIDATOR_MAP.get(normed_key, raise_config_error)
-            value = validator(opt, value)
+            validator = _get_validator(opt, URI_OPTIONS_VALIDATOR_MAP, normed_key=normed_key)
+            validated = validator(opt, value)
         except (ValueError, TypeError, ConfigurationError) as exc:
             if warn:
-                warnings.warn(str(exc))
+                warnings.warn(str(exc), stacklevel=2)
             else:
                 raise
         else:
-            validated_options[get_setter_key(normed_key)] = value
+            validated_options[get_setter_key(normed_key)] = validated
     return validated_options
 
 
-def _esc_coll_name(encrypted_fields, name):
+def _esc_coll_name(encrypted_fields: Mapping[str, Any], name: str) -> Any:
     return encrypted_fields.get("escCollection", f"enxcol_.{name}.esc")
 
 
-def _ecoc_coll_name(encrypted_fields, name):
+def _ecoc_coll_name(encrypted_fields: Mapping[str, Any], name: str) -> Any:
     return encrypted_fields.get("ecocCollection", f"enxcol_.{name}.ecoc")
 
 
@@ -877,9 +901,9 @@ class BaseObject:
 
         if not isinstance(read_preference, _ServerMode):
             raise TypeError(
-                "{!r} is not valid for read_preference. See "
+                f"{read_preference!r} is not valid for read_preference. See "
                 "pymongo.read_preferences for valid "
-                "options.".format(read_preference)
+                "options."
             )
         self.__read_preference = read_preference
 
@@ -910,7 +934,7 @@ class BaseObject:
         """
         return self.__write_concern
 
-    def _write_concern_for(self, session):
+    def _write_concern_for(self, session: Optional[ClientSession]) -> WriteConcern:
         """Read only access to the write concern of this instance or session."""
         # Override this operation's write concern with the transaction's.
         if session and session.in_transaction:
@@ -926,7 +950,7 @@ class BaseObject:
         """
         return self.__read_preference
 
-    def _read_preference_for(self, session):
+    def _read_preference_for(self, session: Optional[ClientSession]) -> _ServerMode:
         """Read only access to the read preference of this instance or session."""
         # Override this operation's read preference with the transaction's.
         if session:
@@ -943,66 +967,74 @@ class BaseObject:
         return self.__read_concern
 
 
-class _CaseInsensitiveDictionary(abc.MutableMapping):
-    def __init__(self, *args, **kwargs):
-        self.__casedkeys = {}
-        self.__data = {}
+class _CaseInsensitiveDictionary(MutableMapping[str, Any]):
+    def __init__(self, *args: Any, **kwargs: Any):
+        self.__casedkeys: dict[str, Any] = {}
+        self.__data: dict[str, Any] = {}
         self.update(dict(*args, **kwargs))
 
-    def __contains__(self, key):
+    def __contains__(self, key: str) -> bool:  # type: ignore[override]
         return key.lower() in self.__data
 
-    def __len__(self):
+    def __len__(self) -> int:
         return len(self.__data)
 
-    def __iter__(self):
+    def __iter__(self) -> Iterator[str]:
         return (key for key in self.__casedkeys)
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return str({self.__casedkeys[k]: self.__data[k] for k in self})
 
-    def __setitem__(self, key, value):
+    def __setitem__(self, key: str, value: Any) -> None:
         lc_key = key.lower()
         self.__casedkeys[lc_key] = key
         self.__data[lc_key] = value
 
-    def __getitem__(self, key):
+    def __getitem__(self, key: str) -> Any:
         return self.__data[key.lower()]
 
-    def __delitem__(self, key):
+    def __delitem__(self, key: str) -> None:
         lc_key = key.lower()
         del self.__casedkeys[lc_key]
         del self.__data[lc_key]
 
-    def __eq__(self, other):
+    def __eq__(self, other: Any) -> bool:
         if not isinstance(other, abc.Mapping):
             return NotImplemented
         if len(self) != len(other):
             return False
-        for key in other:
+        for key in other:  # noqa: SIM110
             if self[key] != other[key]:
                 return False
 
         return True
 
-    def get(self, key, default=None):
+    def get(self, key: str, default: Optional[Any] = None) -> Any:
         return self.__data.get(key.lower(), default)
 
-    def pop(self, key, *args, **kwargs):
+    def pop(self, key: str, *args: Any, **kwargs: Any) -> Any:
         lc_key = key.lower()
         self.__casedkeys.pop(lc_key, None)
         return self.__data.pop(lc_key, *args, **kwargs)
 
-    def popitem(self):
+    def popitem(self) -> tuple[str, Any]:
         lc_key, cased_key = self.__casedkeys.popitem()
         value = self.__data.pop(lc_key)
         return cased_key, value
 
-    def clear(self):
+    def clear(self) -> None:
         self.__casedkeys.clear()
         self.__data.clear()
 
-    def setdefault(self, key, default=None):
+    @overload
+    def setdefault(self, key: str, default: None = None) -> Optional[Any]:
+        ...
+
+    @overload
+    def setdefault(self, key: str, default: Any) -> Any:
+        ...
+
+    def setdefault(self, key: str, default: Optional[Any] = None) -> Optional[Any]:
         lc_key = key.lower()
         if key in self:
             return self.__data[lc_key]
@@ -1011,7 +1043,7 @@ class _CaseInsensitiveDictionary(abc.MutableMapping):
             self.__data[lc_key] = default
             return default
 
-    def update(self, other):
+    def update(self, other: Mapping[str, Any]) -> None:  # type: ignore[override]
         if isinstance(other, _CaseInsensitiveDictionary):
             for key in other:
                 self[other.cased_key(key)] = other[key]
@@ -1019,5 +1051,5 @@ class _CaseInsensitiveDictionary(abc.MutableMapping):
             for key in other:
                 self[key] = other[key]
 
-    def cased_key(self, key):
+    def cased_key(self, key: str) -> Any:
         return self.__casedkeys[key.lower()]
