@@ -20,7 +20,9 @@ import contextlib
 import copy
 import datetime
 import gc
+import logging
 import os
+import re
 import signal
 import socket
 import struct
@@ -29,8 +31,13 @@ import sys
 import threading
 import time
 from typing import Iterable, Type, no_type_check
+from unittest import mock
 from unittest.mock import patch
 
+import pytest
+
+from pymongo.operations import _Op
+
 sys.path[0:0] = [""]
 
 from test import (
@@ -97,13 +104,12 @@ from pymongo.errors import (
 )
 from pymongo.mongo_client import MongoClient
 from pymongo.monitoring import ServerHeartbeatListener, ServerHeartbeatStartedEvent
-from pymongo.pool import _METADATA, Connection, PoolOptions
+from pymongo.pool import _METADATA, ENV_VAR_K8S, Connection, PoolOptions
 from pymongo.read_preferences import ReadPreference
 from pymongo.server_description import ServerDescription
 from pymongo.server_selectors import readable_server_selector, writable_server_selector
 from pymongo.server_type import SERVER_TYPE
 from pymongo.settings import TOPOLOGY_TYPE
-from pymongo.srv_resolver import _HAVE_DNSPYTHON
 from pymongo.topology import _ErrorContext
 from pymongo.topology_description import TopologyDescription
 from pymongo.write_concern import WriteConcern
@@ -122,6 +128,10 @@ class ClientUnitTest(unittest.TestCase):
     def tearDownClass(cls):
         cls.client.close()
 
+    @pytest.fixture(autouse=True)
+    def inject_fixtures(self, caplog):
+        self._caplog = caplog
+
     def test_keyword_arg_defaults(self):
         client = MongoClient(
             socketTimeoutMS=None,
@@ -347,6 +357,15 @@ class ClientUnitTest(unittest.TestCase):
         options = client._MongoClient__options
         self.assertEqual(options.pool_options.metadata, metadata)
 
+    @mock.patch.dict("os.environ", {ENV_VAR_K8S: "1"})
+    def test_container_metadata(self):
+        metadata = copy.deepcopy(_METADATA)
+        metadata["env"] = {}
+        metadata["env"]["container"] = {"orchestrator": "kubernetes"}
+        client = MongoClient("mongodb://foo:27017/?appname=foobar&connect=false")
+        options = client._MongoClient__options
+        self.assertEqual(options.pool_options.metadata["env"], metadata["env"])
+
     def test_kwargs_codec_options(self):
         class MyFloatType:
             def __init__(self, x):
@@ -435,7 +454,6 @@ class ClientUnitTest(unittest.TestCase):
         self.assertEqual(clopts.replica_set_name, "newname")
         self.assertEqual(clopts.read_preference, ReadPreference.SECONDARY_PREFERRED)
 
-    @unittest.skipUnless(_HAVE_DNSPYTHON, "DNS-related tests need dnspython to be installed")
     def test_connection_timeout_ms_propagates_to_DNS_resolver(self):
         # Patch the resolver.
         from pymongo.srv_resolver import _resolve
@@ -525,6 +543,58 @@ class ClientUnitTest(unittest.TestCase):
         self.assertIsInstance(c.options.retry_writes, bool)
         self.assertIsInstance(c.options.retry_reads, bool)
 
+    def test_validate_suggestion(self):
+        """Validate kwargs in constructor."""
+        for typo in ["auth", "Auth", "AUTH"]:
+            expected = f"Unknown option: {typo}. Did you mean one of (authsource, authmechanism, authoidcallowedhosts) or maybe a camelCase version of one? Refer to docstring."
+            expected = re.escape(expected)
+            with self.assertRaisesRegex(ConfigurationError, expected):
+                MongoClient(**{typo: "standard"})  # type: ignore[arg-type]
+
+    @patch("pymongo.srv_resolver._SrvResolver.get_hosts")
+    def test_detected_environment_logging(self, mock_get_hosts):
+        normal_hosts = [
+            "normal.host.com",
+            "host.cosmos.azure.com",
+            "host.docdb.amazonaws.com",
+            "host.docdb-elastic.amazonaws.com",
+        ]
+        srv_hosts = ["mongodb+srv://<test>:<test>@" + s for s in normal_hosts]
+        multi_host = (
+            "host.cosmos.azure.com,host.docdb.amazonaws.com,host.docdb-elastic.amazonaws.com"
+        )
+        with self.assertLogs("pymongo", level="INFO") as cm:
+            for host in normal_hosts:
+                MongoClient(host)
+            for host in srv_hosts:
+                mock_get_hosts.return_value = [(host, 1)]
+                MongoClient(host)
+            MongoClient(multi_host)
+            logs = [record.message for record in cm.records if record.name == "pymongo.client"]
+            self.assertEqual(len(logs), 7)
+
+    @patch("pymongo.srv_resolver._SrvResolver.get_hosts")
+    def test_detected_environment_warning(self, mock_get_hosts):
+        with self._caplog.at_level(logging.WARN):
+            normal_hosts = [
+                "host.cosmos.azure.com",
+                "host.docdb.amazonaws.com",
+                "host.docdb-elastic.amazonaws.com",
+            ]
+            srv_hosts = ["mongodb+srv://<test>:<test>@" + s for s in normal_hosts]
+            multi_host = (
+                "host.cosmos.azure.com,host.docdb.amazonaws.com,host.docdb-elastic.amazonaws.com"
+            )
+            for host in normal_hosts:
+                with self.assertWarns(UserWarning):
+                    MongoClient(host)
+            for host in srv_hosts:
+                mock_get_hosts.return_value = [(host, 1)]
+                with self.assertWarns(UserWarning):
+                    MongoClient(host)
+            with self.assertWarns(UserWarning):
+                MongoClient(multi_host)
+
 
 class TestClient(IntegrationTest):
     def test_multiple_uris(self):
@@ -541,7 +611,7 @@ class TestClient(IntegrationTest):
         with client_knobs(kill_cursor_frequency=0.1):
             # Assert reaper doesn't remove connections when maxIdleTimeMS not set
             client = rs_or_single_client()
-            server = client._get_topology().select_server(readable_server_selector)
+            server = client._get_topology().select_server(readable_server_selector, _Op.TEST)
             with server._pool.checkout() as conn:
                 pass
             self.assertEqual(1, len(server._pool.conns))
@@ -552,7 +622,7 @@ class TestClient(IntegrationTest):
         with client_knobs(kill_cursor_frequency=0.1):
             # Assert reaper removes idle socket and replaces it with a new one
             client = rs_or_single_client(maxIdleTimeMS=500, minPoolSize=1)
-            server = client._get_topology().select_server(readable_server_selector)
+            server = client._get_topology().select_server(readable_server_selector, _Op.TEST)
             with server._pool.checkout() as conn:
                 pass
             # When the reaper runs at the same time as the get_socket, two
@@ -566,7 +636,7 @@ class TestClient(IntegrationTest):
         with client_knobs(kill_cursor_frequency=0.1):
             # Assert reaper respects maxPoolSize when adding new connections.
             client = rs_or_single_client(maxIdleTimeMS=500, minPoolSize=1, maxPoolSize=1)
-            server = client._get_topology().select_server(readable_server_selector)
+            server = client._get_topology().select_server(readable_server_selector, _Op.TEST)
             with server._pool.checkout() as conn:
                 pass
             # When the reaper runs at the same time as the get_socket,
@@ -580,7 +650,7 @@ class TestClient(IntegrationTest):
         with client_knobs(kill_cursor_frequency=0.1):
             # Assert reaper has removed idle socket and NOT replaced it
             client = rs_or_single_client(maxIdleTimeMS=500)
-            server = client._get_topology().select_server(readable_server_selector)
+            server = client._get_topology().select_server(readable_server_selector, _Op.TEST)
             with server._pool.checkout() as conn_one:
                 pass
             # Assert that the pool does not close connections prematurely.
@@ -597,14 +667,15 @@ class TestClient(IntegrationTest):
     def test_min_pool_size(self):
         with client_knobs(kill_cursor_frequency=0.1):
             client = rs_or_single_client()
-            server = client._get_topology().select_server(readable_server_selector)
+            server = client._get_topology().select_server(readable_server_selector, _Op.TEST)
             self.assertEqual(0, len(server._pool.conns))
 
             # Assert that pool started up at minPoolSize
             client = rs_or_single_client(minPoolSize=10)
-            server = client._get_topology().select_server(readable_server_selector)
+            server = client._get_topology().select_server(readable_server_selector, _Op.TEST)
             wait_until(
-                lambda: len(server._pool.conns) == 10, "pool initialized with 10 connections"
+                lambda: len(server._pool.conns) == 10,
+                "pool initialized with 10 connections",
             )
 
             # Assert that if a socket is closed, a new one takes its place
@@ -620,7 +691,7 @@ class TestClient(IntegrationTest):
         # Use high frequency to test _get_socket_no_auth.
         with client_knobs(kill_cursor_frequency=99999999):
             client = rs_or_single_client(maxIdleTimeMS=500)
-            server = client._get_topology().select_server(readable_server_selector)
+            server = client._get_topology().select_server(readable_server_selector, _Op.TEST)
             with server._pool.checkout() as conn:
                 pass
             self.assertEqual(1, len(server._pool.conns))
@@ -634,7 +705,7 @@ class TestClient(IntegrationTest):
 
             # Test that connections are reused if maxIdleTimeMS is not set.
             client = rs_or_single_client()
-            server = client._get_topology().select_server(readable_server_selector)
+            server = client._get_topology().select_server(readable_server_selector, _Op.TEST)
             with server._pool.checkout() as conn:
                 pass
             self.assertEqual(1, len(server._pool.conns))
@@ -1682,7 +1753,6 @@ class TestClient(IntegrationTest):
         with self.assertRaises(InvalidOperation):
             coll.insert_many([{} for _ in range(5)])
 
-    @unittest.skipUnless(_HAVE_DNSPYTHON, "DNS-related tests need dnspython to be installed")
     def test_service_name_from_kwargs(self):
         client = MongoClient(
             "mongodb+srv://user:password@test22.test.build.10gen.cc",
@@ -1703,7 +1773,6 @@ class TestClient(IntegrationTest):
         )
         self.assertEqual(client._topology_settings.srv_service_name, "customname")
 
-    @unittest.skipUnless(_HAVE_DNSPYTHON, "DNS-related tests need dnspython to be installed")
     def test_srv_max_hosts_kwarg(self):
         client = MongoClient("mongodb+srv://test1.test.build.10gen.cc/")
         self.assertGreater(len(client.topology_description.server_descriptions()), 1)
@@ -1714,11 +1783,6 @@ class TestClient(IntegrationTest):
         )
         self.assertEqual(len(client.topology_description.server_descriptions()), 2)
 
-    @unittest.skipIf(_HAVE_DNSPYTHON, "dnspython must not be installed")
-    def test_srv_no_dnspython_error(self):
-        with self.assertRaisesRegex(ConfigurationError, 'The "dnspython" module must be'):
-            MongoClient("mongodb+srv://test1.test.build.10gen.cc/")
-
     @unittest.skipIf(
         client_context.load_balancer or client_context.serverless,
         "loadBalanced clients do not run SDAM",
@@ -2160,7 +2224,7 @@ class TestMongoClientFailover(MockClientTest):
 
         # But it can reconnect.
         c.revive_host("a:1")
-        c._get_topology().select_servers(writable_server_selector)
+        c._get_topology().select_servers(writable_server_selector, _Op.TEST)
         self.assertEqual(c.address, ("a", 1))
 
     def _test_network_error(self, operation_callback):
@@ -2183,7 +2247,7 @@ class TestMongoClientFailover(MockClientTest):
             # Set host-specific information so we can test whether it is reset.
             c.set_wire_version_range("a:1", 2, 6)
             c.set_wire_version_range("b:2", 2, 7)
-            c._get_topology().select_servers(writable_server_selector)
+            c._get_topology().select_servers(writable_server_selector, _Op.TEST)
             wait_until(lambda: len(c.nodes) == 2, "connect")
 
             c.kill_host("a:1")
