@@ -17,6 +17,8 @@ from __future__ import annotations
 
 import sys
 
+from pymongo.operations import _Op
+
 sys.path[0:0] = [""]
 
 from test import client_knobs, unittest
@@ -30,11 +32,12 @@ from pymongo.hello import Hello, HelloCompat
 from pymongo.monitor import Monitor
 from pymongo.pool import PoolOptions
 from pymongo.read_preferences import ReadPreference, Secondary
+from pymongo.server import Server
 from pymongo.server_description import ServerDescription
 from pymongo.server_selectors import any_server_selector, writable_server_selector
 from pymongo.server_type import SERVER_TYPE
 from pymongo.settings import TopologySettings
-from pymongo.topology import Topology, _ErrorContext
+from pymongo.topology import Topology, _ErrorContext, _filter_servers
 from pymongo.topology_description import TOPOLOGY_TYPE
 
 
@@ -169,7 +172,7 @@ class TestSingleServerTopology(TopologyTest):
 
             # Can't select a server while the only server is of type Unknown.
             with self.assertRaisesRegex(ConnectionFailure, "No servers found yet"):
-                t.select_servers(any_server_selector, server_selection_timeout=0)
+                t.select_servers(any_server_selector, _Op.TEST, server_selection_timeout=0)
 
             got_hello(t, address, hello_response)
 
@@ -178,7 +181,7 @@ class TestSingleServerTopology(TopologyTest):
 
             # No matter whether the server is writable,
             # select_servers() returns it.
-            s = t.select_server(writable_server_selector)
+            s = t.select_server(writable_server_selector, _Op.TEST)
             self.assertEqual(server_type, s.description.server_type)
 
             # Topology type single is always readable and writable regardless
@@ -216,7 +219,7 @@ class TestSingleServerTopology(TopologyTest):
 
         t = create_mock_topology(monitor_class=TestMonitor)
         self.addCleanup(t.close)
-        s = t.select_server(writable_server_selector)
+        s = t.select_server(writable_server_selector, _Op.TEST)
         self.assertEqual(125, s.description.round_trip_time)
 
         round_trip_time = 25
@@ -231,7 +234,7 @@ class TestSingleServerTopology(TopologyTest):
 
         def raises_err():
             try:
-                t.select_server(writable_server_selector, server_selection_timeout=0.1)
+                t.select_server(writable_server_selector, _Op.TEST, server_selection_timeout=0.1)
             except ConnectionFailure:
                 return True
             else:
@@ -452,7 +455,9 @@ class TestMultiServerTopology(TopologyTest):
         # Discovering a replica set without the setName supplied by the user
         # is not yet supported by MongoClient, but Topology can do it.
         topology_settings = SetNameDiscoverySettings(
-            seeds=[address], pool_class=MockPool, monitor_class=DummyMonitor  # type: ignore[arg-type]
+            seeds=[address],
+            pool_class=MockPool,  # type: ignore[arg-type]
+            monitor_class=DummyMonitor,  # type: ignore[arg-type]
         )
 
         t = Topology(topology_settings)
@@ -480,7 +485,9 @@ class TestMultiServerTopology(TopologyTest):
         # Discovering a replica set without the setName supplied by the user
         # is not yet supported by MongoClient, but Topology can do it.
         topology_settings = SetNameDiscoverySettings(
-            seeds=[address], pool_class=MockPool, monitor_class=DummyMonitor  # type: ignore[arg-type]
+            seeds=[address],
+            pool_class=MockPool,  # type: ignore[arg-type]
+            monitor_class=DummyMonitor,  # type: ignore[arg-type]
         )
 
         t = Topology(topology_settings)
@@ -530,7 +537,7 @@ class TestMultiServerTopology(TopologyTest):
 
         self.assertEqual(server.description.min_wire_version, 1)
         self.assertEqual(server.description.max_wire_version, 6)
-        t.select_servers(any_server_selector)
+        t.select_servers(any_server_selector, _Op.TEST)
 
         # Incompatible.
         got_hello(
@@ -547,7 +554,7 @@ class TestMultiServerTopology(TopologyTest):
         )
 
         try:
-            t.select_servers(any_server_selector)
+            t.select_servers(any_server_selector, _Op.TEST)
         except ConfigurationError as e:
             # Error message should say which server failed and why.
             self.assertEqual(
@@ -573,7 +580,7 @@ class TestMultiServerTopology(TopologyTest):
         )
 
         try:
-            t.select_servers(any_server_selector)
+            t.select_servers(any_server_selector, _Op.TEST)
         except ConfigurationError as e:
             # Error message should say which server failed and why.
             self.assertEqual(
@@ -589,7 +596,7 @@ class TestMultiServerTopology(TopologyTest):
         t = create_mock_topology(seeds=["a", "b"], replica_set_name="rs")
 
         def write_batch_size():
-            s = t.select_server(writable_server_selector)
+            s = t.select_server(writable_server_selector, _Op.TEST)
             return s.description.max_write_batch_size
 
         got_hello(
@@ -681,6 +688,23 @@ class TestMultiServerTopology(TopologyTest):
         self.assertNotIn(("a", 27017), t.description.server_descriptions())
         self.assertEqual(t.description.topology_type_name, "Unknown")
 
+    def test_filtered_server_selection(self):
+        s1 = Server(ServerDescription(("localhost", 27017)), pool=object(), monitor=object())  # type: ignore[arg-type]
+        s2 = Server(ServerDescription(("localhost2", 27017)), pool=object(), monitor=object())  # type: ignore[arg-type]
+        servers = [s1, s2]
+
+        result = _filter_servers(servers, deprioritized_servers=[s2])
+        self.assertEqual(result, [s1])
+
+        result = _filter_servers(servers, deprioritized_servers=[s1, s2])
+        self.assertEqual(result, servers)
+
+        result = _filter_servers(servers, deprioritized_servers=[])
+        self.assertEqual(result, servers)
+
+        result = _filter_servers(servers)
+        self.assertEqual(result, servers)
+
 
 def wait_for_primary(topology):
     """Wait for a Topology to discover a writable server.
@@ -693,7 +717,7 @@ def wait_for_primary(topology):
 
     def get_primary():
         try:
-            return topology.select_server(writable_server_selector, 0)
+            return topology.select_server(writable_server_selector, _Op.TEST, 0)
         except ConnectionFailure:
             return None
 
@@ -749,7 +773,7 @@ class TestTopologyErrors(TopologyTest):
         # The third hello call (the immediate retry) happens sometime soon
         # after the failed check triggered by request_check_all. Wait until
         # the server becomes known again.
-        server = t.select_server(writable_server_selector, 0.250)
+        server = t.select_server(writable_server_selector, _Op.TEST, 0.250)
         self.assertEqual(SERVER_TYPE.Standalone, server.description.server_type)
         self.assertEqual(3, hello_count[0])
 
@@ -763,13 +787,13 @@ class TestTopologyErrors(TopologyTest):
         t = create_mock_topology(monitor_class=TestMonitor)
         self.addCleanup(t.close)
         with self.assertRaisesRegex(ConnectionFailure, "internal error"):
-            t.select_server(any_server_selector, server_selection_timeout=0.5)
+            t.select_server(any_server_selector, _Op.TEST, server_selection_timeout=0.5)
 
 
 class TestServerSelectionErrors(TopologyTest):
     def assertMessage(self, message, topology, selector=any_server_selector):
         with self.assertRaises(ConnectionFailure) as context:
-            topology.select_server(selector, server_selection_timeout=0)
+            topology.select_server(selector, _Op.TEST, server_selection_timeout=0)
 
         self.assertIn(message, str(context.exception))
 
